@@ -1,7 +1,7 @@
 # -*- mode: Perl -*-
 package MRTG_lib;
 ###################################################################
-# MRTG 2.16.3  Support library MRTG_lib.pm
+# MRTG 2.17.2  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <tobi@oetiker.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -18,12 +18,6 @@ require 5.005;
 use strict;
 use vars qw($OS $SL $PS @EXPORT @ISA $VERSION %timestrpospattern);
 
-if (eval {local $SIG{__DIE__}; require Net_SNMP_util} ) {
-	import Net_SNMP_util;
-} else {
-	require SNMP_util; 
-        import SNMP_util;
-}
 
 my %mrtgrules;
 
@@ -156,6 +150,12 @@ $VERSION = 2.100016;
        'conversioncode' =>
        [sub{-r $_[0]}, sub{"Cannot read conversion code file $_[0]"}],
 
+       # Check for an environment setting for RRDCACHED_ADDRESS
+       # Steve Shipway, Sep 2010
+       'rrdcached' =>
+#       [sub{(($_[0] =~ /^unix:(\S+)/)and(-w $1))}, sub{"Currently, only UNIX domain sockets are supported for RRDCached, and must exist and be writeable."}],
+       [sub{1},sub{"Internal Error"}],
+
        # Per Router CFG
        'target[]' => 
        [sub{1}, sub{"Internal Error"}], #will test this later
@@ -463,14 +463,25 @@ sub readcfg ($$$$;$$) {
         chomp;
         my $line = $.;
         if (/^include:\s*(.*?\S)\s*$/i){
-                push @filestack, $file;
-                push @handstack, $hand;
+                my $newhandle;
+                my @nextfiles;
                 $nextfile = $1;
-                local *FH;
-                open (FH, $nextfile)
-                 || open (FH, ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile)
-                 || do { die "ERROR: unable to open include file: $nextfile\n"};
-                $hand = *FH;
+                if( $nextfile =~ /\*/ ) {
+                    @nextfiles = glob( $nextfile );
+                    @nextfiles = glob( ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile )
+                        if(!@nextfiles);	
+                } else { 
+                    $nextfile =  ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile 
+                        if(!-r $nextfile);
+                    @nextfiles = ( $nextfile ); 
+                }
+                foreach $nextfile ( @nextfiles ) {
+                    open my $newhandle, '<', $nextfile or die "ERROR: unable to open include file: $nextfile\n";
+                    push @handstack, $hand;
+                    push @filestack, $file;
+                    $hand = $newhandle;
+                    $file = $nextfile;
+                }
                 next;
         }
 
@@ -631,6 +642,26 @@ sub readcfg ($$$$;$$) {
     }
 
     close (CFG);
+
+	# Check for an environment setting for RRDCACHED_ADDRESS
+	# Steve Shipway, Sep 2010
+	if( $ENV{RRDCACHED_ADDRESS} and not exists $cfg->{ rrdcached } ) {
+		warn("WARNING: Using environment variable RRDCACHED_ADDRESS\n");
+		$cfg->{ rrdcached } = $ENV{RRDCACHED_ADDRESS};
+        quickcheck('rrdcached',0,$ENV{RRDCACHED_ADDRESS},'Environment variable RRDCACHED_ADDRESS','n/a',\%mrtgrules);
+	}
+	if( exists $cfg->{ rrdcached } ) {
+        warn ("WARNING: You are running with RRDCached enabled (".$cfg->{ rrdcached }.").  This will disable all Threshold checking, since RRDCached does not support updatev and an update/fetch will cancel out the caching benefits.\n");
+        if( $cfg->{ rrdcached } !~ /^unix:/ ) {
+            warn("WARNING: You are running RRDCached in TCP mode.  This means that it will use its own Base Directory instead of WorkDir for storing the RRD files.  Also, changes to MaxBytes and DS Type will not be actioned after the RRD file has been created.\n");
+        }
+	}
+    if ($cfg->{enablesnmpv3} and $cfg->{enablesnmpv3} eq 'yes' and eval {local $SIG{__DIE__}; require Net_SNMP_util} ) {
+        import Net_SNMP_util;
+    } else {
+        require SNMP_util;
+        import SNMP_util;
+    }
 }
 
 # quick checks
@@ -644,16 +675,16 @@ sub quickcheck ($$$$$$) {
             return 1;
         } else {
             if ($second) {
-                die "ERROR: CFG Error in \"$first\[$second\]\", line $line: ".
+                die "ERROR: CFG Error in \"$first\[$second\]\", file $file line $line: ".
                   &{$rules->{$first.$braces}[1]}($arg)."\n\n"; 
             } else {
-                die "ERROR: CFG Error in \"$first\", line $line: ".
+                die "ERROR: CFG Error in \"$first\", file $file line $line: ".
                   &{$rules->{$first.$braces}[1]}($arg)."\n\n"; 
             } 
         }
     }
-    die "ERROR: CFG Error Unknown Option \"$first\" on line $line or above.\n".
-      "           Check doc/reference.txt for Help\n\n";
+    die "ERROR: CFG Error Unknown Option \"$first\" in file $file on line $line or above.\n".
+      "           Check doc/mrtg-reference.txt for Help\n\n";
 }
 
 # complex config checks
@@ -731,8 +762,10 @@ sub cfgcheck ($$$$;$) {
 	warn ("WARNING: If \"ThreshMailSender\" is defined, then \"ThreshMailServer\" must be defined too.\n");
         $error = "yes";
     }
- 
-    $cfg->{threshhyst} = 0.1 unless $cfg->{threshhyst};
+    # default ThreshHyst to 0.1 if ThreshDir is defined
+    if ($cfg->{threshdir}){
+        $cfg->{threshhyst} = 0.1 unless $cfg->{threshhyst};
+    }
     # build relativ path from htmldir to image dir.
     my @htmldir = split /\Q${MRTG_lib::SL}\E+/, $$cfg{htmldir};
     my @imagedir =  split /\Q${MRTG_lib::SL}\E+/, $$cfg{imagedir};
@@ -763,16 +796,16 @@ sub cfgcheck ($$$$;$) {
     if(defined $$cfg{libadd}){
         ensureSL(\$$cfg{libadd});
         debug('eval',"libadd $$cfg{libadd}\n");
-	local $SIG{__DIE__};
+    	local $SIG{__DIE__};
         eval "use lib qw( $$cfg{libadd} )";
-	my @match;
-	foreach my $dir (@INC){
-		push @match, $dir if -f "$dir/RRDs.pm";
-	}
-	warn "WARN: found several copies of RRDs.pm in your path: ".
-        (join ", ", @match)." I will be using $match[0]. This could ".
-	"be a problem if this is an old copy and you think I would be using a newer one!\n"
-		if $#match > 0;
+    	my @match;
+	    foreach my $dir (@INC){
+		    push @match, $dir if -f "$dir/RRDs.pm";
+    	}
+	    warn "WARN: found several copies of RRDs.pm in your path: ".
+            (join ", ", @match)." I will be using $match[0]. This could ".
+        	"be a problem if this is an old copy and you think I would be using a newer one!\n"
+		    if $#match > 0;
     }
     $$cfg{logformat} = 'rateup' unless defined $$cfg{logformat};
 
@@ -836,6 +869,7 @@ sub cfgcheck ($$$$;$) {
         die "ERROR: File $cfg->{ conversioncode } conversion code evaluation failed\n$@\n"
             unless eval $code;
     }
+
     my $thresh_error;
 
     foreach $rou (@$routers) {
@@ -1076,6 +1110,12 @@ sub cfgcheck ($$$$;$) {
         if (! defined $$rcfg{'yscale'}{$rou}) {
             $$rcfg{'yscale'}{$rou} = 1.0;
         }
+        if (defined $$rcfg{'options'}{'pngdate'}{$rou}) {
+            $$rcfg{'timestrpos'}{$rou} = 'RU';
+            $$rcfg{'timestrfmt'}{$rou} = "%Y-%m-%d %H:%M %Z";
+
+            delete $$rcfg{'options'}{'pntdate'}{$rou}
+        }
         if (! defined $$rcfg{'timestrpos'}{$rou}) {
             $$rcfg{'timestrpos'}{$rou} = 'NO';
         }
@@ -1178,7 +1218,7 @@ sub demonize_me ($) {
      if (my_type() eq 'VIO'){
         $main::Cleanfile3 = $pidfile;
 
-        print "MRTG detached. PID=".system(P_DETACH,$^X." ".$0." ".$cfgfile);
+        print "MRTG detached. PID=".system(P_DETACH(),$^X." ".$0." ".$cfgfile);
         exit;
      }
     } else {
@@ -1186,7 +1226,8 @@ sub demonize_me ($) {
            if (defined $pidfile && open(READPID, "<$pidfile")){
                if (not eof READPID) {
                    chomp(my $input = <READPID>);    # read process id in pidfile
-                   if ($input && kill 0 => $input) {# oops - the pid actually exists
+                   my ($pid) = $input =~ /^(\d+)$/; # to improve taint-safe code
+                   if ($pid && kill 0 => $pid) {# oops - the pid actually exists
                         die "ERROR: I Quit! Another copy of mrtg seems to be running. Check $pidfile\n";
                    }
                }
@@ -1685,7 +1726,7 @@ sub populateconfcache ($$$$$) {
 	next if $confcache->{___deadhosts}{$hostkey} and time - $confcache->{___deadhosts}{$hostkey} < 300;
 	$SNMP_Session::errmsg = undef;
 	$Net_SNMP_util::ErrorMessage = undef;
-	@ret = snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, $node);
+	@ret = &main::snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, $node);
 	unless ( $SNMP_Session::errmsg or $Net_SNMP_util::ErrorMessage){
 	    foreach my $ret (@ret)
 	      {
@@ -1713,7 +1754,7 @@ sub populateconfcache ($$$$$) {
     }
     $SNMP_Session::errmsg = undef;
     $Net_SNMP_util::ErrorMessage = undef;
-    @ret = snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, "ifPhysAddress");
+    @ret = &main::snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, "ifPhysAddress");
     unless ( $SNMP_Session::errmsg or $Net_SNMP_util::ErrorMessage){
 	foreach my $ret (@ret)
 	  {
